package com.kfzs.duanduan.datashare;

import com.kfzs.duanduan.data.graph.provider.player.User;
import com.kfzs.duanduan.datashare.provider.session.SessionBean;

/**
 * <pre>
 *     sinlov
 *
 *     /\__/\
 *    /`    '\
 *  ≈≈≈ 0  0 ≈≈≈ Hello world!
 *    \  --  /
 *   /        \
 *  /          \
 * |            |
 *  \  ||  ||  /
 *   \_oo__oo_/≡≡≡≡≡≡≡≡o
 *
 * </pre>
 * Created by sinlov on 17/3/6.
 */
public interface CertificationDog {

    int RUNTIME_ERROR = 1 << 1;
    int PARAMS_ERROR = 2 << 1;
    int BUNDLE_ERROR = 3 << 1;
    int CERTIFICATION_ERROR = 4 << 1;
    int CERTIFICATION_CODE_NOT_HAVE = 5 << 1;
    int TARGET_APP_CAN_NOT_FIND = 6 << 1;

    void onCertificationCodeCallBack(User user, String gamePackageName, String gameCode, String extrasString);

    void onCertificationSuccess();

    void onCertificationCancel();

    void onLoginJob();

    void onError(int errorCode, String msg);
}
